@@ -273,6 +273,8 @@ class Shodan:
         self.tools = self.Tools(self)
         self.stream = Stream(key, proxies=proxies)
         self._session = requests.Session()
+        self.api_rate_limit = 1  # Requests per second
+        self._api_query_time = None
         if proxies:
             self._session.proxies.update(proxies)
             self._session.trust_env = False
@@ -297,6 +299,11 @@ class Shodan:
             'exploits': self.base_exploits_url,
         }.get(service, 'shodan')
 
+        # Wait for API rate limit
+        if self._api_query_time is not None and self.api_rate_limit > 0:
+            while (1.0 / self.api_rate_limit) + self._api_query_time >= time.time():
+                time.sleep(0.1 / self.api_rate_limit)
+
         # Send the request
         try:
             method = method.lower()
@@ -308,6 +315,7 @@ class Shodan:
                 data = self._session.delete(base_url + function, params=params)
             else:
                 data = self._session.get(base_url + function, params=params)
+            self._api_query_time = time.time()
         except Exception:
             raise APIError('Unable to connect to Shodan')
 
@@ -328,6 +336,8 @@ class Shodan:
             raise APIError(error)
         elif data.status_code == 403:
             raise APIError('Access denied (403 Forbidden)')
+        elif data.status_code == 502:
+            raise APIError('Bad Gateway (502)')
 
         # Parse the text into JSON
         try:
@@ -377,7 +387,7 @@ class Shodan:
             params['history'] = history
         if minify:
             params['minify'] = minify
-        return self._request('/shodan/host/%s' % ','.join(ips), params)
+        return self._request('/shodan/host/{}'.format(','.join(ips)), params)
 
     def info(self):
         """Returns information about the current API key, such as a list of add-ons
@@ -468,7 +478,7 @@ class Shodan:
 
         :returns: A dictionary with general information about the scan, including its status in getting processed.
         """
-        return self._request('/shodan/scan/%s' % scan_id, {})
+        return self._request('/shodan/scan/{}'.format(scan_id), {})
 
     def search(self, query, page=1, limit=None, offset=None, facets=None, minify=True):
         """Search the SHODAN database.
@@ -677,7 +687,7 @@ class Shodan:
     def alerts(self, aid=None, include_expired=True):
         """List all of the active alerts that the user created."""
         if aid:
-            func = '/shodan/alert/%s/info' % aid
+            func = '/shodan/alert/{}/info'.format(aid)
         else:
             func = '/shodan/alert/info'
 
@@ -689,7 +699,7 @@ class Shodan:
 
     def delete_alert(self, aid):
         """Delete the alert with the given ID."""
-        func = '/shodan/alert/%s' % aid
+        func = '/shodan/alert/{}'.format(aid)
 
         response = api_request(self.api_key, func, params={}, method='delete',
                                proxies=self._session.proxies)
@@ -711,8 +721,14 @@ class Shodan:
         """Disable the given trigger on the alert."""
         return self._request('/shodan/alert/{}/trigger/{}'.format(aid, trigger), {}, method='delete')
 
-    def ignore_alert_trigger_notification(self, aid, trigger, ip, port):
+    def ignore_alert_trigger_notification(self, aid, trigger, ip, port, vulns=None):
         """Ignore trigger notifications for the provided IP and port."""
+        # The "vulnerable" and "vulnerable_unverified" triggers let you specify specific vulnerabilities
+        # to ignore. If a user provides a "vulns" list and specifies on of those triggers then we'll use
+        # a different API endpoint.
+        if trigger in ('vulnerable', 'vulnerable_unverified') and vulns and isinstance(vulns, list):
+            return self._request('/shodan/alert/{}/trigger/{}/ignore/{}:{}/{}'.format(aid, trigger, ip, port, ','.join(vulns)), {}, method='put')
+        
         return self._request('/shodan/alert/{}/trigger/{}/ignore/{}:{}'.format(aid, trigger, ip, port), {}, method='put')
 
     def unignore_alert_trigger_notification(self, aid, trigger, ip, port):
